@@ -23,12 +23,16 @@ from media.track.fileTrack import FileTrack
 
 def getTrack(file):
     """ Return a Track created from an mp3 file """
-    id3     = ID3(file)
     track   = FileTrack(file)
     mp3File = MP3(file)
 
     track.setLength(int(round(mp3File.info.length)))
 
+    try:
+        id3 = ID3(file)
+    except:
+        return track
+
     try:    track.setTitle(str(id3['TIT2']))
     except: pass
 
