@@ -26,7 +26,6 @@ from media.track.cdTrack import CDTrack
 
 MOD_INFO = ('Audio CD', _('Audio CD'), _('Play audio discs'), ('DiscID', 'CDDB'), False, True)
 
-
 MOD_L10N            = MOD_INFO[modules.MODINFO_L10N]
 PREFS_DFT_DEVICE    = '/dev/cdrom'
 PREFS_DFT_USE_CDDB  = True
@@ -65,6 +64,7 @@ class AudioCD(modules.ThreadedModule):
         self.tree     = extTreeview.ExtTreeView(columns, True)
         self.popup    = None
         self.cfgWin   = None
+        self.expName  = MOD_L10N
         self.scrolled = gtk.ScrolledWindow()
         self.cacheDir = os.path.join(consts.dirCfg, MOD_INFO[modules.MODINFO_NAME])
         # Explorer
@@ -77,7 +77,7 @@ class AudioCD(modules.ThreadedModule):
         self.tree.connect('drag-data-get',              self.onDragDataGet)
         self.tree.connect('key-press-event',            self.onKeyPressed)
         self.tree.connect('exttreeview-button-pressed', self.onButtonPressed)
-        modules.postMsg(consts.MSG_CMD_EXPLORER_ADD, {'modName': MOD_L10N, 'expName': MOD_L10N, 'icon': consts.icoCdrom, 'widget': self.scrolled})
+        modules.postMsg(consts.MSG_CMD_EXPLORER_ADD, {'modName': MOD_L10N, 'expName': self.expName, 'icon': consts.icoCdrom, 'widget': self.scrolled})
 
 
     def onModUnloaded(self):
@@ -174,6 +174,10 @@ class AudioCD(modules.ThreadedModule):
         artist, album = disc.split(' / ')
         self.tree.setItem((0,), ROW_NAME, rowName)
 
+        # Update the explorer name
+        modules.postMsg(consts.MSG_CMD_EXPLORER_RENAME, {'modName': MOD_L10N, 'expName': self.expName, 'newExpName': disc})
+        self.expName = disc
+
         # Optional information
         try:    date = int(cddb['DYEAR'].strip().decode('iso-8859-15', 'replace'))
         except: date = None
@@ -236,6 +240,8 @@ class AudioCD(modules.ThreadedModule):
         except Exception, err:
             if err[0] == 123:
                 self.tree.replaceContent([(consts.icoCdrom, _('No disc found'), None)])
+                modules.postMsg(consts.MSG_CMD_EXPLORER_RENAME, {'modName': MOD_L10N, 'expName': self.expName, 'newExpName': MOD_L10N})
+                self.expName = MOD_L10N
             else:
                 logger.error('[%s] Unable to read device\n\n%s' % (MOD_INFO[modules.MODINFO_NAME], traceback.format_exc()))
             return
@@ -252,7 +258,7 @@ class AudioCD(modules.ThreadedModule):
     def reloadDisc(self):
         """ Reload the disc """
         # Sort of hack, to be sure that the reloading is done in the thread's code and not in the GTK main loop
-        self.postMsg(consts.MSG_EVT_EXPLORER_CHANGED, {'modName': MOD_L10N, 'expName': MOD_L10N})
+        self.postMsg(consts.MSG_EVT_EXPLORER_CHANGED, {'modName': MOD_L10N, 'expName': self.expName})
 
 
     # --== Message handler ==--
