@@ -29,8 +29,9 @@ MOD_INFO = ('File Explorer', _('File Explorer'), _('Browse your file system'), [
 MOD_L10N = MOD_INFO[modules.MODINFO_L10N]
 
 # Default preferences
-PREFS_DEFAULT_MEDIA_FOLDERS     = {_('Home'): consts.dirBaseUsr}    # List of media folders that are used as roots for the file explorer
-PREFS_DEFAULT_SHOW_HIDDEN_FILES = False                             # True if hidden files should be shown
+PREFS_DEFAULT_MEDIA_FOLDERS     = {_('Home'): consts.dirBaseUsr, _('Root'): '/'}    # List of media folders that are used as roots for the file explorer
+PREFS_DEFAULT_ADD_BY_FILENAME   = False                                             # True if files should be added to the playlist by their filename
+PREFS_DEFAULT_SHOW_HIDDEN_FILES = False                                             # True if hidden files should be shown
 
 
 # The format of a row in the treeview
@@ -45,10 +46,9 @@ PREFS_DEFAULT_SHOW_HIDDEN_FILES = False                             # True if hi
 # The possible types for a node of the tree
 (
     TYPE_DIR,   # A directory
-    TYPE_PLIST, # A playlist
     TYPE_FILE,  # A media file
     TYPE_NONE   # A fake item, used to display a '+' in front of a directory when needed
-) = range(4)
+) = range(3)
 
 
 class FileExplorer(modules.Module):
@@ -61,14 +61,14 @@ class FileExplorer(modules.Module):
 
     def onModLoaded(self):
         """ The module has been loaded """
-        self.trees           = {}     # A [tree, state] per root folder
-        self.popup           = None
+        self.tree            = None
         self.cfgWin          = None
         self.folders         = prefs.get(__name__, 'media-folders', PREFS_DEFAULT_MEDIA_FOLDERS)
         self.scrolled        = gtk.ScrolledWindow()
-        self.currFolder      = None
+        self.currRoot        = None
+        self.addByFilename   = prefs.get(__name__, 'add-by-filename',  PREFS_DEFAULT_ADD_BY_FILENAME)
         self.showHiddenFiles = prefs.get(__name__, 'show-hidden-files', PREFS_DEFAULT_SHOW_HIDDEN_FILES)
-        # Explorer
+
         self.scrolled.set_shadow_type(gtk.SHADOW_IN)
         self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         self.scrolled.show()
@@ -77,44 +77,65 @@ class FileExplorer(modules.Module):
             modules.postMsg(consts.MSG_CMD_EXPLORER_ADD, {'modName': MOD_L10N, 'expName': name, 'icon': None, 'widget': self.scrolled})
 
 
-    def createTree(self, rootFolder):
-        """ Return a new tree for the given root folder """
-        columns = (('',   [(gtk.CellRendererPixbuf(), gtk.gdk.Pixbuf), (gtk.CellRendererText(), TYPE_STRING)], True),
-                   (None, [(None, TYPE_INT)],                                                                  False),
-                   (None, [(None, TYPE_STRING)],                                                               False))
-
-        tree = extTreeview.ExtTreeView(columns, True)
-        tree.setIsDraggableFunc(self.isPlayable)
-        tree.setDNDSources([consts.DND_TARGETS[consts.DND_DAP_URI]])
-        # GTK handlers
-        tree.connect('drag-data-get',              self.onDragDataGet)
-        tree.connect('key-press-event',            self.onKeyPressed)
-        tree.connect('exttreeview-row-expanded',   self.onRowExpanded)
-        tree.connect('exttreeview-row-collapsed',  self.onRowCollapsed)
-        tree.connect('exttreeview-button-pressed', self.onMouseButton)
-
-        return tree
-
-
     def onModUnloaded(self):
         """ The module is going to be unloaded """
         prefs.set(__name__, 'media-folders',     self.folders)
+        prefs.set(__name__, 'add-by-filename',   self.addByFilename)
         prefs.set(__name__, 'show-hidden-files', self.showHiddenFiles)
 
+        # Save the state of the current tree if needed
+        if self.currRoot is not None:
+            savedStates = prefs.get(__name__, 'saved-states', {})
+            savedStates[self.currRoot] = {
+                                            'tree-state':     self.dumpTree(),
+                                            'selected-paths': self.tree.getSelectedPaths(),
+                                            'vscrollbar-pos': self.scrolled.get_vscrollbar().get_value(),
+                                            'hscrollbar-pos': self.scrolled.get_hscrollbar().get_value(),
+                                         }
+            prefs.set(__name__, 'saved-states', savedStates)
+
+
+    def dumpTree(self, path=None):
+        """ Recursively dump the given tree starting at path (None for the root of the tree) """
+        list = []
+
+        for child in self.tree.iterChildren(path):
+            row = self.tree.getRow(child)
+
+            if self.tree.getNbChildren(child) == 0: grandChildren = None
+            elif self.tree.row_expanded(child):     grandChildren = self.dumpTree(child)
+            else:                                   grandChildren = []
+
+            list.append([(row[ROW_NAME], row[ROW_TYPE], row[ROW_FULLPATH]), grandChildren])
+
+        return list
+
+
+    def restoreTree(self, dump, parent=None):
+        """ Recursively restore the dump under the given parent (None for the root of the tree) """
+        for item in dump:
+            (name, type, path) = item[0]
+
+            if type == TYPE_FILE:
+                self.tree.appendRow((consts.icoMediaFile, name, TYPE_FILE, path), parent)
+            else:
+                newNode = self.tree.appendRow((consts.icoDir, name, TYPE_DIR, path), parent)
+
+                if item[1] is not None:
+                    fakeChild = self.tree.appendRow((consts.icoDir, '', TYPE_NONE, ''), newNode)
+
+                    if len(item[1]) != 0:
+                        # We must expand the row before adding the real children, but this works only if there is already at least one child
+                        self.tree.expandRow(newNode)
+                        self.restoreTree(item[1], newNode)
+                        self.tree.removeRow(fakeChild)
+
 
     def sortKey(self, row):
         """ Key function used to compare two rows of the tree """
         return row[ROW_NAME].lower()
 
 
-    def isPlayable(self):
-        """ Only playable rows can be dragged """
-        for row in self.trees[self.currFolder][0].iterSelectedRows():
-            if row[ROW_TYPE] in (TYPE_FILE, TYPE_PLIST) or (row[ROW_TYPE] == TYPE_DIR and row[ROW_PIXBUF] == consts.icoMediaDir):
-                return True
-        return False
-
-
     def setShowHiddenFiles(self, showHiddenFiles):
         """ Show/hide hidden files """
         if showHiddenFiles != self.showHiddenFiles:
@@ -123,68 +144,51 @@ class FileExplorer(modules.Module):
                 self.cfgWin.getWidget('chk-hidden').set_active(showHiddenFiles)
 
             self.showHiddenFiles = showHiddenFiles
-            self.refresh(self.trees[self.currFolder][0])
+            self.refresh()
 
 
-    def getFilesFromPaths(self, tree, paths):
-        """
-            Return a list of playable files from:
-                * The list 'paths' if it is not None
-                * The selected rows if paths is None
-        """
-        if paths is None: return [row[ROW_FULLPATH] for row in tree.getSelectedRows() if row[ROW_TYPE] != TYPE_NONE]
-        else:             return [row[ROW_FULLPATH] for row in tree.getRows(paths) if row[ROW_TYPE] != TYPE_NONE]
-
-
-    def playPaths(self, tree, paths, replace):
+    def play(self, replace, path=None):
         """
             Replace/extend the tracklist
-            If the list 'paths' is None, use the current selection
+            If 'path' is None, use the current selection
         """
-        if self.isPlayable():
-            tracks = media.getTracks(self.getFilesFromPaths(tree, paths))
-            if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET, {'tracks': tracks, 'playNow': True})
-            else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks})
+        if path is None: tracks = media.getTracks([row[ROW_FULLPATH] for row in self.tree.getSelectedRows()], self.addByFilename)
+        else:            tracks = media.getTracks([self.tree.getRow(path)[ROW_FULLPATH]], self.addByFilename)
+
+        if replace: modules.postMsg(consts.MSG_CMD_TRACKLIST_SET, {'tracks': tracks, 'playNow': True})
+        else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks})
 
 
     def renameFolder(self, oldName, newName):
         """ Rename a folder """
-        # Remove immediately the tree from the scrolled window if needed
-        # We can't do that later, because we have no longer access to the old name after that
-        if self.currFolder == oldName:
-            self.scrolled.remove(self.trees[oldName][0])
-            self.currFolder = None
-
-        # Rename the root folder
         self.folders[newName] = self.folders[oldName]
         del self.folders[oldName]
 
-        # Move the tree associated to that folder, if any
-        if oldName in self.trees:
-            self.trees[newName] = self.trees[oldName]
-            del self.trees[oldName]
+        savedStates = prefs.get(__name__, 'saved-states', {})
+        if oldName in savedStates:
+            savedStates[newName] = savedStates[oldName]
+            del savedStates[oldName]
+            prefs.set(__name__, 'saved-states', savedStates)
 
-        modules.postMsg(consts.MSG_CMD_EXPLORER_REMOVE, {'modName': MOD_L10N, 'expName': oldName})
-        modules.postMsg(consts.MSG_CMD_EXPLORER_ADD,    {'modName': MOD_L10N, 'expName': newName, 'icon': None, 'widget': self.scrolled})
+        modules.postMsg(consts.MSG_CMD_EXPLORER_RENAME,   {'modName': MOD_L10N, 'expName': oldName, 'newExpName': newName})
 
 
     # --== Tree management ==--
 
 
-    def __startLoading(self, tree, row):
+    def startLoading(self, row):
         """ Tell the user that the contents of row is being loaded """
-        name = tree.getItem(row, ROW_NAME)
-        # TODO Put this style somewhere, we're using it in multiple places
-        tree.setItem(row, ROW_NAME, '%s  <span size="smaller" foreground="#909090">%s</span>' % (name, _('loading...')))
+        name = self.tree.getItem(row, ROW_NAME)
+        self.tree.setItem(row, ROW_NAME, '%s  <span size="smaller" foreground="#909090">%s</span>' % (name, _('loading...')))
 
 
-    def __stopLoading(self, tree, row):
+    def stopLoading(self, row):
         """ Tell the user that the contents of row has been loaded"""
-        name  = tree.getItem(row, ROW_NAME)
+        name  = self.tree.getItem(row, ROW_NAME)
         index = name.find('<')
 
         if index != -1:
-            tree.setItem(row, ROW_NAME, name[:index-2])
+            self.tree.setItem(row, ROW_NAME, name[:index-2])
 
 
     def getDirContents(self, directory):
@@ -200,7 +204,7 @@ class FileExplorer(modules.Module):
                 if media.isSupported(file):
                     mediaFiles.append((consts.icoMediaFile, cgi.escape(unicode(file, errors='replace')), TYPE_FILE, path))
                 elif playlist.isSupported(file):
-                    playlists.append((consts.icoMediaFile, cgi.escape(unicode(file, errors='replace')), TYPE_PLIST, path))
+                    playlists.append((consts.icoMediaFile, cgi.escape(unicode(file, errors='replace')), TYPE_FILE, path))
 
         playlists.sort(key=self.sortKey)
         mediaFiles.sort(key=self.sortKey)
@@ -209,102 +213,108 @@ class FileExplorer(modules.Module):
         return (directories, playlists, mediaFiles)
 
 
-    def exploreDir(self, tree, parent, directory, fakeChild=None):
+    def exploreDir(self, parent, directory, fakeChild=None):
         """
             List the contents of the given directory and append it to the tree as a child of parent
             If fakeChild is not None, remove it when the real contents has been loaded
         """
         directories, playlists, mediaFiles = self.getDirContents(directory)
 
-        tree.appendRows(directories, parent)
-        tree.appendRows(playlists,   parent)
-        tree.appendRows(mediaFiles,  parent)
+        self.tree.appendRows(directories, parent)
+        self.tree.appendRows(playlists,   parent)
+        self.tree.appendRows(mediaFiles,  parent)
 
         if fakeChild is not None:
-            tree.removeRow(fakeChild)
+            self.tree.removeRow(fakeChild)
 
-        idle_add(self.updateChildren(tree, parent).next)
+        idle_add(self.updateDirNodes(parent).next)
 
 
-    def updateChildren(self, tree, parent):
-        """ This generator updates (e.g., icon, fake child for directories) all children of the given parent """
-        for child in tree.iterChildren(parent):
-            # Only directories need to be updated and since they are always on top, we can stop if we find something else
-            if tree.getItem(child, ROW_TYPE) != TYPE_DIR:
+    def updateDirNodes(self, parent):
+        """ This generator updates the directory nodes, based on whether they should be expandable """
+        for child in self.tree.iterChildren(parent):
+            # Only directories need to be updated and since they all come first, we can stop as soon as we find something else
+            if self.tree.getItem(child, ROW_TYPE) != TYPE_DIR:
                 break
 
             # Make sure it's readable
-            directory = tree.getItem(child, ROW_FULLPATH)
-            if not os.access(directory, os.R_OK | os.X_OK):
-                continue
-
-            hasContent      = False
-            hasMediaContent = False
-            for (file, path) in tools.listDir(directory, self.showHiddenFiles):
-                if isdir(path):
-                    hasContent = True
-                elif isfile(path) and (media.isSupported(file) or playlist.isSupported(file)):
-                    hasContent      = True
-                    hasMediaContent = True
-                    break
+            directory  = self.tree.getItem(child, ROW_FULLPATH)
+            hasContent = False
+            if os.access(directory, os.R_OK | os.X_OK):
+                for (file, path) in tools.listDir(directory, self.showHiddenFiles):
+                    if isdir(path) or (isfile(path) and (media.isSupported(file) or playlist.isSupported(file))):
+                        hasContent = True
+                        break
 
             # Append/remove children if needed
-            if hasContent and tree.getNbChildren(child) == 0:      tree.appendRow((consts.icoDir, '', TYPE_NONE, ''), child)
-            elif not hasContent and tree.getNbChildren(child) > 0: tree.removeAllChildren(child)
-
-            # Change the icon based on whether the folder contains some media content
-            if hasMediaContent: tree.setItem(child, ROW_PIXBUF, consts.icoMediaDir)
-            else:               tree.setItem(child, ROW_PIXBUF, consts.icoDir)
+            if hasContent and self.tree.getNbChildren(child) == 0:      self.tree.appendRow((consts.icoDir, '', TYPE_NONE, ''), child)
+            elif not hasContent and self.tree.getNbChildren(child) > 0: self.tree.removeAllChildren(child)
 
             yield True
 
-
         if parent is not None:
-            self.__stopLoading(tree, parent)
+            self.stopLoading(parent)
 
         yield False
 
 
-    def refresh(self, tree, treePath=None):
+    def refresh(self, treePath=None):
         """ Refresh the tree, starting from treePath """
-        if treePath is None: directory = self.folders[self.currFolder]
-        else:                directory = tree.getItem(treePath, ROW_FULLPATH)
+        if treePath is None: directory = self.folders[self.currRoot]
+        else:                directory = self.tree.getItem(treePath, ROW_FULLPATH)
 
         directories, playlists, mediaFiles = self.getDirContents(directory)
 
-        disk       = directories + playlists + mediaFiles
-        diskIndex  = 0
-        childIndex = 0
+        disk                   = directories + playlists + mediaFiles
+        diskIndex              = 0
+        childIndex             = 0
+        childLeftIntentionally = False
+
         while diskIndex < len(disk):
-            file    = disk[diskIndex]
-            rowPath = tree.getChild(treePath, childIndex)
+            rowPath = self.tree.getChild(treePath, childIndex)
 
-            # We've reached the end of the tree, append the file and switch to the next one
+            # Did we reach the end of the tree?
             if rowPath is None:
-                tree.appendRow(file, treePath)
-                continue
+                break
+
+            file      = disk[diskIndex]
+            cmpResult = cmp(self.sortKey(self.tree.getRow(rowPath)), self.sortKey(file))
 
-            cmpResult = cmp(self.sortKey(tree.getRow(rowPath)), self.sortKey(file))
             if cmpResult < 0:
-                tree.removeRow(rowPath)
+                # We can't remove the only child left, to prevent the node from being closed automatically
+                if self.tree.getNbChildren(treePath) == 1:
+                    childLeftIntentionally = True
+                    break
+
+                self.tree.removeRow(rowPath)
             else:
                 if cmpResult > 0:
-                    tree.insertRowBefore(file, treePath, rowPath)
+                    self.tree.insertRowBefore(file, treePath, rowPath)
                 diskIndex  += 1
                 childIndex += 1
 
         # If there are tree rows left, all the corresponding files are no longer there
-        while childIndex < tree.getNbChildren(treePath):
-            tree.removeRow(tree.getChild(treePath, childIndex))
+        if not childLeftIntentionally:
+            while childIndex < self.tree.getNbChildren(treePath):
+                self.tree.removeRow(self.tree.getChild(treePath, childIndex))
+
+        # Disk files left?
+        while diskIndex < len(disk):
+            self.tree.appendRow(disk[diskIndex], treePath)
+            diskIndex += 1
+
+        # Deprecated child left? (must be done after the addition of left disk files)
+        if childLeftIntentionally:
+            self.tree.removeRow(self.tree.getChild(treePath, 0))
 
         # Update nodes' appearance
         if len(directories) != 0:
-            idle_add(self.updateChildren(tree, treePath).next)
+            idle_add(self.updateDirNodes(treePath).next)
 
         # Recursively refresh expanded rows
-        for child in tree.iterChildren(treePath):
-            if tree.row_expanded(child):
-                idle_add(self.refresh, tree, child)
+        for child in self.tree.iterChildren(treePath):
+            if self.tree.row_expanded(child):
+                idle_add(self.refresh, child)
 
 
     # --== GTK handlers ==--
@@ -316,49 +326,71 @@ class FileExplorer(modules.Module):
             self.onShowPopupMenu(tree, event.button, event.time, path)
         elif path is not None:
             if event.button == 2:
-                 self.playPaths(tree, [path], False)
+                self.play(False, path)
             elif event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
-                if   tree.getItem(path, ROW_PIXBUF) != consts.icoDir: self.playPaths(tree, None, True)
+                if   tree.getItem(path, ROW_PIXBUF) != consts.icoDir: self.play(True)
                 elif tree.row_expanded(path):                         tree.collapse_row(path)
                 else:                                                 tree.expand_row(path, False)
 
 
     def onShowPopupMenu(self, tree, button, time, path):
         """ Show a popup menu """
-        if self.popup is None:
-            self.popup = tools.loadGladeFile('FileExplorerMenu.glade')
-            self.popup.get_widget('item-refresh').connect('activate', lambda widget: self.refresh(tree))
-            self.popup.get_widget('item-collapse').connect('activate', lambda widget: tree.collapse_all())
-            self.popup.get_widget('item-add').connect('activate', lambda widget: self.playPaths(tree, None, False))
-            self.popup.get_widget('item-play').connect('activate', lambda widget: self.playPaths(tree, None, True))
-            self.popup.get_widget('item-hidden').connect('toggled', lambda item: self.setShowHiddenFiles(item.get_active()))
-            self.popup.get_widget('menu-popup').show_all()
+        popup = gtk.Menu()
 
-        playable = path is not None and tree.getItem(path, ROW_PIXBUF) != consts.icoDir
+        # Play selection
+        play = gtk.ImageMenuItem(gtk.STOCK_MEDIA_PLAY)
+        play.set_sensitive(path is not None)
+        play.connect('activate', lambda widget: self.play(True))
+        popup.append(play)
 
-        self.popup.get_widget('item-add').set_sensitive(playable)
-        self.popup.get_widget('item-play').set_sensitive(playable)
-        self.popup.get_widget('item-hidden').set_active(self.showHiddenFiles)
-        self.popup.get_widget('menu-popup').popup(None, None, None, button, time)
+        # Add selection
+        add = gtk.ImageMenuItem(gtk.STOCK_ADD)
+        add.set_sensitive(path is not None)
+        add.connect('activate', lambda widget: self.play(False))
+        popup.append(add)
+
+        popup.append(gtk.SeparatorMenuItem())
+
+        # Collapse all nodes
+        collapse = gtk.ImageMenuItem(_('Collapse all'))
+        collapse.set_image(gtk.image_new_from_stock(gtk.STOCK_CLEAR, gtk.ICON_SIZE_MENU))
+        collapse.connect('activate', lambda widget: tree.collapse_all())
+        popup.append(collapse)
+
+        # Refresh the library
+        refresh = gtk.ImageMenuItem(gtk.STOCK_REFRESH)
+        refresh.connect('activate', lambda widget: self.refresh())
+        popup.append(refresh)
+
+        popup.append(gtk.SeparatorMenuItem())
+
+        # Show hidden files
+        hidden = gtk.CheckMenuItem(_('Show hidden files'))
+        hidden.set_active(self.showHiddenFiles)
+        hidden.connect('toggled', lambda item: self.setShowHiddenFiles(item.get_active()))
+        popup.append(hidden)
+
+        popup.show_all()
+        popup.popup(None, None, None, button, time)
 
 
     def onKeyPressed(self, tree, event):
         """ A key has been pressed """
         keyname = gtk.gdk.keyval_name(event.keyval)
 
-        if keyname == 'F5':       self.refresh(tree)
+        if keyname == 'F5':       self.refresh()
         elif keyname == 'plus':   tree.expandRows()
         elif keyname == 'Left':   tree.collapseRows()
         elif keyname == 'Right':  tree.expandRows()
         elif keyname == 'minus':  tree.collapseRows()
         elif keyname == 'space':  tree.switchRows()
-        elif keyname == 'Return': self.playPaths(tree, None, True)
+        elif keyname == 'Return': self.play(True)
 
 
     def onRowExpanded(self, tree, path):
         """ Replace the fake child by the real children """
-        idle_add(self.__startLoading, tree, path)
-        idle_add(self.exploreDir, tree, path, tree.getItem(path, ROW_FULLPATH), tree.getChild(path, 0))
+        self.startLoading(path)
+        idle_add(self.exploreDir, path, tree.getItem(path, ROW_FULLPATH), tree.getChild(path, 0))
 
 
     def onRowCollapsed(self, tree, path):
@@ -369,7 +401,7 @@ class FileExplorer(modules.Module):
 
     def onDragDataGet(self, tree, context, selection, info, time):
         """ Provide information about the data being dragged """
-        selection.set('text/uri-list', 8, ' '.join([urllib.pathname2url(file) for file in self.getFilesFromPaths(tree, None)]))
+        selection.set('text/uri-list', 8, ' '.join([urllib.pathname2url(file) for file in [row[ROW_FULLPATH] for row in tree.getSelectedRows()]]))
 
 
    # --== Message handler ==--
@@ -377,33 +409,56 @@ class FileExplorer(modules.Module):
 
     def handleMsg(self, msg, params):
         """ Handle messages sent to this module """
-        if msg == consts.MSG_EVT_EXPLORER_CHANGED and params['modName'] == MOD_L10N and self.currFolder != params['expName']:
-            newFolder = params['expName']
+        if msg == consts.MSG_EVT_EXPLORER_CHANGED and params['modName'] == MOD_L10N and self.currRoot != params['expName']:
+            newRoot = params['expName']
 
-            # Create a tree for this root folder if there's not already one
-            if newFolder not in self.trees:
-                self.trees[newFolder] = [self.createTree(newFolder), None]
+            # Create the tree if needed (this is done only the very first time)
+            if self.tree is None:
+                columns = (('',   [(gtk.CellRendererPixbuf(), gtk.gdk.Pixbuf), (gtk.CellRendererText(), TYPE_STRING)], True),
+                           (None, [(None, TYPE_INT)],                                                                  False),
+                           (None, [(None, TYPE_STRING)],                                                               False))
 
-            tree, state = self.trees[newFolder]
+                self.tree = extTreeview.ExtTreeView(columns, True)
 
-            # Make sure this tree is the current child of the scrolled window
-            if self.currFolder is not None:
-                currTree = self.trees[self.currFolder][0]
-                self.trees[self.currFolder] = [currTree, currTree.saveState(ROW_NAME)]
-                self.scrolled.remove(currTree)
-            self.scrolled.add(tree)
+                self.scrolled.add(self.tree)
+                self.tree.setDNDSources([consts.DND_TARGETS[consts.DND_DAP_URI]])
+                self.tree.connect('drag-data-get', self.onDragDataGet)
+                self.tree.connect('key-press-event', self.onKeyPressed)
+                self.tree.connect('exttreeview-button-pressed', self.onMouseButton)
+                self.tree.connect('exttreeview-row-collapsed', self.onRowCollapsed)
+                self.expandedHandler = self.tree.connect('exttreeview-row-expanded', self.onRowExpanded)
 
-            # Display the contents of the root folder
-            self.currFolder = params['expName']
+            savedStates = prefs.get(__name__, 'saved-states', {})
 
-            # If the tree has just been created, we need to explore the root folder to populate the tree
-            if state is not None:
-                # FIXME This doesn't work, the scrollbar stays at the top...
-                idle_add(tree.restoreState, state, ROW_NAME)
+            # Save the current state if needed
+            if self.currRoot is not None:
+                savedStates[self.currRoot] = {
+                                                'tree-state':     self.dumpTree(),
+                                                'selected-paths': self.tree.getSelectedPaths(),
+                                                'vscrollbar-pos': self.scrolled.get_vscrollbar().get_value(),
+                                                'hscrollbar-pos': self.scrolled.get_hscrollbar().get_value(),
+                                             }
+                prefs.set(__name__, 'saved-states', savedStates)
+                self.tree.clear()
+
+            self.currRoot = newRoot
+
+            if newRoot not in savedStates:
+                self.exploreDir(None, self.folders[self.currRoot])
+                if len(self.tree) != 0:
+                    self.tree.scroll_to_cell(0)
             else:
-                self.exploreDir(tree, None, self.folders[self.currFolder])
-                if len(tree) != 0:
-                    tree.scroll_to_cell(0)
+                savedState = savedStates[newRoot]
+
+                self.tree.disconnect(self.expandedHandler)
+                self.restoreTree(savedState['tree-state'])
+                self.expandedHandler  = self.tree.connect('exttreeview-row-expanded', self.onRowExpanded)
+
+                idle_add(self.scrolled.get_vscrollbar().set_value, savedState['vscrollbar-pos'])
+                idle_add(self.scrolled.get_hscrollbar().set_value, savedState['hscrollbar-pos'])
+                idle_add(self.tree.selectPaths, savedState['selected-paths'])
+                idle_add(self.refresh)
+
         elif msg == consts.MSG_EVT_APP_STARTED:
             self.onModLoaded()
         elif msg == consts.MSG_EVT_APP_QUIT:
@@ -439,6 +494,7 @@ class FileExplorer(modules.Module):
         if not self.cfgWin.isVisible():
             self.populateFolderList()
             self.cfgWin.getWidget('chk-hidden').set_active(self.showHiddenFiles)
+            self.cfgWin.getWidget('chk-add-by-filename').set_active(self.addByFilename)
             self.cfgWin.getWidget('btn-ok').grab_focus()
 
         self.cfgWin.show()
@@ -477,9 +533,9 @@ class FileExplorer(modules.Module):
                 del self.folders[name]
 
                 # Remove the tree, if any, from the scrolled window
-                if self.currFolder == name:
+                if self.currRoot == name:
                     self.scrolled.remove(self.trees[name][0])
-                    self.currFolder = None
+                    self.currRoot = None
 
                 # Remove the tree associated to this root folder, if any
                 if name in self.trees:
@@ -518,6 +574,7 @@ class FileExplorer(modules.Module):
         """ The user has clicked on the OK button """
         self.cfgWin.hide()
         self.setShowHiddenFiles(self.cfgWin.getWidget('chk-hidden').get_active())
+        self.addByFilename = self.cfgWin.getWidget('chk-add-by-filename').get_active()
 
 
     def onHelp(self, btn):
