@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import cgi, collections, gtk, gui, media, modules, os, shutil, sys, tools
+import cgi, collections, gtk, gui, media, modules, os, random, shutil, sys, tools
 
 from gui                   import fileChooser, help, questionMsgBox, extTreeview, extListview, progressDlg, selectPath
 from tools                 import consts, prefs, pickleLoad, pickleSave
@@ -98,7 +98,6 @@ class Library(modules.Module):
     def onAppStarted(self):
         """ This is the real initialization function, called when the module has been loaded """
         self.tree      = None
-        self.popup     = None
         self.currLib   = None
         self.cfgWindow = None
         self.libraries = prefs.get(__name__, 'libraries',  PREFS_DEFAULT_LIBRARIES)
@@ -359,22 +358,80 @@ class Library(modules.Module):
         else:       modules.postMsg(consts.MSG_CMD_TRACKLIST_ADD, {'tracks': tracks})
 
 
+    def pickAlbumArtist(self, tree, artistPath):
+        """ Pick an album at random of the given artist and play it """
+        # Expanding the artist row populates it, so that we can then pick an album at random
+        tree.expandRow(artistPath)
+        albumPath = artistPath + (random.randint(0, tree.getNbChildren(artistPath)-1), )
+
+        # Select the random album and play it
+        tree.get_selection().unselect_all()
+        tree.get_selection().select_path(albumPath)
+        tree.scroll(albumPath)
+        self.playPaths(tree, [albumPath], True)
+
+
+    def pickAlbumLibrary(self, tree):
+        """ Pick an album at random in the library and play it """
+        # Pick an artist at random (make sure not to select an alphabetical header)
+        while True:
+            path = (random.randint(0, tree.getCount()-1), )
+            if tree.getItem(path, ROW_TYPE) == TYPE_ARTIST:
+                break
+
+        self.pickAlbumArtist(tree, path)
+
+
     def showPopupMenu(self, tree, button, time, path):
         """ Show a popup menu """
-        if self.popup is None:
-            self.popup = tools.loadGladeFile('LibraryMenu.glade')
-            self.popup.get_widget('menu-popup').show_all()
-            # Connect handlers
-            self.popup.get_widget('item-collapse').connect('activate', lambda widget: self.tree.collapse_all())
-            self.popup.get_widget('item-play').connect('activate',     lambda widget: self.playPaths(tree, None, True))
-            self.popup.get_widget('item-add').connect('activate',      lambda widget: self.playPaths(tree, None, False))
-            self.popup.get_widget('item-refresh').connect('activate',  lambda widget: idle_add(self.refreshLibrary(None, self.currLib, self.libraries[self.currLib][LIB_PATH]).next))
-
+        popup    = gtk.Menu()
         playable = path is not None and tree.getItem(path, ROW_TYPE) != TYPE_NONE
-        self.popup.get_widget('item-add').set_sensitive(playable)
-        self.popup.get_widget('item-play').set_sensitive(playable)
 
-        self.popup.get_widget('menu-popup').popup(None, None, None, button, time)
+        # Play selection
+        play = gtk.ImageMenuItem(gtk.STOCK_MEDIA_PLAY)
+        play.set_sensitive(playable)
+        play.connect('activate', lambda widget: self.playPaths(tree, None, True))
+        popup.append(play)
+
+        # Add selection
+        add = gtk.ImageMenuItem(gtk.STOCK_ADD)
+        add.set_sensitive(playable)
+        add.connect('activate', lambda widget: self.playPaths(tree, None, False))
+        popup.append(add)
+
+        popup.append(gtk.SeparatorMenuItem())
+
+        # Collapse all nodes
+        collapse = gtk.ImageMenuItem(_('Collapse all'))
+        collapse.set_image(gtk.image_new_from_stock(gtk.STOCK_CLEAR, gtk.ICON_SIZE_MENU))
+        collapse.connect('activate', lambda widget: self.tree.collapse_all())
+        popup.append(collapse)
+
+        # Refresh the library
+        refresh = gtk.ImageMenuItem(gtk.STOCK_REFRESH)
+        refresh.connect('activate', lambda widget: idle_add(self.refreshLibrary(None, self.currLib, self.libraries[self.currLib][LIB_PATH]).next))
+        popup.append(refresh)
+
+        # Randomness
+        randomness     = gtk.Menu()
+        randomnessItem = gtk.ImageMenuItem(_('Randomness'))
+        randomnessItem.get_image().set_from_icon_name('stock_shuffle', gtk.ICON_SIZE_MENU)
+        randomnessItem.set_submenu(randomness)
+        popup.append(randomnessItem)
+
+        # Random album of the selected artist
+        if path is not None and tree.getItem(path, ROW_TYPE) == TYPE_ARTIST:
+            album = gtk.MenuItem(_('Pick an album of %(artist)s' % {'artist': tree.getItem(path, ROW_NAME).replace('&amp;', '&')}))
+            album.connect('activate', lambda widget: self.pickAlbumArtist(tree, path))
+            randomness.append(album)
+
+        # Random album in the entire library
+        album = gtk.MenuItem(_('Pick an album in the library'))
+        album.connect('activate', lambda widget: self.pickAlbumLibrary(tree))
+        randomness.append(album)
+
+        popup.show_all()
+        popup.popup(None, None, None, button, time)
 
 
     def loadLibrary(self, tree, name):
@@ -582,8 +639,7 @@ class Library(modules.Module):
         newPath = os.path.join(ROOT_PATH, newName)
         shutil.move(oldPath, newPath)
 
-        modules.postMsg(consts.MSG_CMD_EXPLORER_REMOVE, {'modName': MOD_L10N, 'expName': oldName})
-        modules.postMsg(consts.MSG_CMD_EXPLORER_ADD,    {'modName': MOD_L10N, 'expName': newName, 'icon': None, 'widget': self.scrolled})
+        modules.postMsg(consts.MSG_CMD_EXPLORER_RENAME, {'modName': MOD_L10N, 'expName': oldName, 'newExpName': newName})
 
 
     def onRenameLibrary(self, btn):
