@@ -16,16 +16,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import gtk, os.path, shutil
+import gtk, os.path, random, shutil, time
 
 from gettext import gettext as _
 
-
-def __sureFile(filename):
-    """ Return filename if it exists, or None """
-    if os.path.exists(filename): return filename
-    else:                        return None
-
+# --- Not a constant, but it fits well here
+random.seed(int(time.time()))
 
 # --- Miscellaneous
 socketTimeout = 10
@@ -33,7 +29,7 @@ socketTimeout = 10
 
 # --- Strings
 appName      = 'Decibel Audio Player'
-appVersion   = '1.00'
+appVersion   = '1.02'
 appNameShort = 'decibel-audio-player'
 
 
@@ -77,17 +73,15 @@ if not os.path.exists(dirLog): os.mkdir(dirLog)
 fileImgIcon16  = os.path.join(dirPix, 'decibel-audio-player-16.png')
 fileImgIcon24  = os.path.join(dirPix, 'decibel-audio-player-24.png')
 fileImgIcon32  = os.path.join(dirPix, 'decibel-audio-player-32.png')
-fileImgIcon48  = os.path.join(dirPix, 'decibel-audio-player.png')
+fileImgIcon48  = os.path.join(dirPix, 'decibel-audio-player-48.png')
 fileImgIcon64  = os.path.join(dirPix, 'decibel-audio-player-64.png')
 fileImgIcon128 = os.path.join(dirPix, 'decibel-audio-player-128.png')
 
 
 # --- Files
-fileLog        = os.path.join(dirLog, 'log')
-filePrefs      = os.path.join(dirCfg, 'prefs.txt')
-fileLicense    = os.path.join(dirDoc, 'LICENCE')
-fileImgRepeat  = __sureFile('/usr/share/icons/gnome/22x22/status/media-playlist-repeat.png')
-fileImgShuffle = __sureFile('/usr/share/icons/gnome/22x22/status/media-playlist-shuffle.png')
+fileLog     = os.path.join(dirLog, 'log')
+filePrefs   = os.path.join(dirCfg, 'prefs.txt')
+fileLicense = os.path.join(dirDoc, 'LICENCE')
 
 
 # --- DBus constants
@@ -143,18 +137,31 @@ DND_TARGETS = {
               }
 
 
+# --- View modes
+(
+    VIEW_MODE_FULL,
+    VIEW_MODE_MINI,
+    VIEW_MODE_NANO,
+    VIEW_MODE_PLAYLIST,
+) = range(4)
+
+
 # --- Message that can be sent/received by modules
 # --- A message is always associated with a (potentially empty) dictionnary containing its parameters
 (
     # --== COMMANDS ==--
 
     # GStreamer player
-    MSG_CMD_PLAY,             # Play a resource     Parameters: 'uri'
-    MSG_CMD_STOP,             # Stop playing        Parameters:
-    MSG_CMD_SEEK,             # Jump to a position  Parameters: 'seconds'
-    MSG_CMD_SET_VOLUME,       # Change the volume   Parameters: 'value'
-    MSG_CMD_BUFFER,           # Buffer a file       Parameters: 'filename'
-    MSG_CMD_TOGGLE_PAUSE,     # Toggle play/pause   Parameters:
+    MSG_CMD_PLAY,             # Play a resource                            Parameters: 'uri'
+    MSG_CMD_STOP,             # Stop playing                               Parameters:
+    MSG_CMD_SEEK,             # Jump to a position                         Parameters: 'seconds'
+    MSG_CMD_SET_VOLUME,       # Change the volume                          Parameters: 'value'
+    MSG_CMD_BUFFER,           # Buffer a file                              Parameters: 'filename'
+    MSG_CMD_TOGGLE_PAUSE,     # Toggle play/pause                          Parameters:
+    MSG_CMD_ENABLE_EQZ,       # Enable the equalizer                       Parameters:
+    MSG_CMD_SET_EQZ_LVLS,     # Set the levels of the 10-bands equalizer   Parameters: 'lvls'
+    MSG_CMD_ENABLE_RG,        # Enable ReplayGain                          Parameters:
+    MSG_CMD_DISABLE_RG,       # Disable ReplayGain                         Parameters:
 
     # Tracklist
     MSG_CMD_NEXT,              # Play the next track       Parameters:
@@ -167,6 +174,7 @@ DND_TARGETS = {
     # Explorers
     MSG_CMD_EXPLORER_ADD,      # Add a new explorer    Parameters: 'modName', 'expName', 'icon', 'widget'
     MSG_CMD_EXPLORER_REMOVE,   # Remove an explorer    Parameters: 'modName', 'expName'
+    MSG_CMD_EXPLORER_RENAME,   # Rename an explorer    Parameters: 'modName', 'expName', 'newExpName'
 
     # Covers
     MSG_CMD_SET_COVER,         # Cover file for the given track     Parameters: 'track', 'pathThumbnail', 'pathFullSize'
@@ -203,4 +211,4 @@ DND_TARGETS = {
 
     # End value
     MSG_END_VALUE
-) = range(32)
+) = range(37)
